@@ -22,15 +22,16 @@
 
 #include "editorscene.h"
 #include "gradientdialog.h"
-#include "global.h"
+#include "qsstvglobal.h"
 #include "ui_textform.h"
 #include "gallerywidget.h"
+#include "utils/qjp2io.h"
 
 editorScene::editorScene(QGraphicsView *parent)  : QGraphicsScene(parent)
 {
 	contextMenu=new QMenu();
 	arrange = new QMenu( "Arrange");
-	arrange->setTearOffEnabled(TRUE);
+	arrange->setTearOffEnabled(true);
 	arrange->addAction("Forward",this,SLOT(slotSendForward()));
 	arrange->addAction("Backward",this,SLOT(slotSendBackward()));
 	arrange->addAction("Bring to front",this,SLOT(slotBringToFront()));
@@ -46,7 +47,7 @@ editorScene::editorScene(QGraphicsView *parent)  : QGraphicsScene(parent)
 	contextMenu->addAction("Lock",this,SLOT(slotLock()));
 	contextMenu->addAction("Unlock",this,SLOT(slotUnlock()));
 	zMax=0;
-	pasted=FALSE;
+	pasted=false;
 	copyItem=NULL;
 	mode=MOVE;
 	imageType=NONE;
@@ -70,14 +71,14 @@ editorScene::~editorScene()
 
 bool editorScene::load(QFile &f)
 {
-  bool borderSet=FALSE;
+  bool borderSet=false;
 	QImage im;
 	itemBase *item;
 	quint32 magic;
 	QString version;
 	quint16 streamVersion;
 	int type;
-	if(!f.open(QIODevice::ReadOnly)) return FALSE;
+	if(!f.open(QIODevice::ReadOnly)) return false;
 	QDataStream str(&f);
   str >> magic;
 	
@@ -87,19 +88,19 @@ bool editorScene::load(QFile &f)
 			f.reset();
 			if(im.load(&f,0))
 				{
-          addToLog("image loaded",DBEDIT);
+          addToLog("image loaded",LOGEDIT);
 					imageType=FLATIMAGE;
 					setImage(&im);
           border=QRect(0,0,im.width(),im.height());
-          borderSet=TRUE;
+          borderSet=true;
 					f.close();
-					return TRUE;
+					return true;
 				}
 			else
 				{
-          addToLog("image failed to load",DBEDIT);
+          addToLog("image failed to load",LOGEDIT);
 					f.close();
-					return FALSE;
+					return false;
 				}
 		}
 	imageType=TEMPLATE;
@@ -130,7 +131,7 @@ bool editorScene::load(QFile &f)
             item=new itemReplayImage(contextMenu);
           break;
         case itemBase::SBORDER:
-            borderSet=TRUE;
+            borderSet=true;
             item=new itemImage(contextMenu);
             item->load(str);
             border=item->rect();
@@ -138,9 +139,9 @@ bool editorScene::load(QFile &f)
             continue;
         break;
 					default:
-            addToLog("Error in datastream",DBEDIT);
+            addToLog("Error in datastream",LOGEDIT);
 						f.close();
-						return FALSE;
+						return false;
 					break;
 				}
 			item->load(str);
@@ -154,34 +155,41 @@ bool editorScene::load(QFile &f)
   if(!borderSet) border=QRectF(0,0,320,256);
   addToLog(QString("border position %1,%2 size: %3 x %4 border set=%5")
                .arg(border.topLeft().x()).arg(border.topLeft().y())
-               .arg(border.width()).arg(border.height()).arg(borderSet),DBEDIT);
+               .arg(border.width()).arg(border.height()).arg(borderSet),LOGEDIT);
   f.close();
   setSceneRect(border);
-	return TRUE;
+	return true;
 }
 
-QImage *editorScene::renderImage()
+QImage *editorScene::renderImage(int w,int h)
 {
   clearSelection();
   if (localImage!=NULL) delete localImage;
   //border=sceneRect();
-  localImage=new QImage(border.width(),border.height(),QImage::Format_ARGB32);
-  addToLog(QString("editorScene: pre-render size: %1 x %2").arg(localImage->size().width()).arg(localImage->size().height()),DBEDIT);
+  if(w==0)
+    {
+      localImage=new QImage(border.width(),border.height(),QImage::Format_ARGB32_Premultiplied);
+    }
+  else
+    {
+      localImage=new QImage(w,h,QImage::Format_ARGB32_Premultiplied);
+    }
+  addToLog(QString("editorScene: pre-render size: %1 x %2").arg(localImage->size().width()).arg(localImage->size().height()),LOGEDIT);
   QPainter painter(localImage);
   painter.setRenderHint(QPainter::Antialiasing);
   //setSceneRect(0,0,localImage->width(),localImage->height());
   localImage->fill(0);
   render(&painter);
-  addToLog(QString("editor: post-render size: %1 x %2").arg(localImage->size().width()).arg(localImage->size().height()),DBEDIT);
+  addToLog(QString("editor: post-render size: %1 x %2").arg(localImage->size().width()).arg(localImage->size().height()),LOGEDIT);
   return localImage;
 }
 
-void editorScene::flattenImage()
+void editorScene::flattenImage(int w,int h)
 {
   if (localImage!=NULL) delete localImage;
   setSceneRect(border);
 //  border=sceneRect();
-  localImage=new QImage(border.width(),border.height(),QImage::Format_ARGB32);
+  localImage=new QImage(w,h,QImage::Format_ARGB32_Premultiplied);
   convertText();
   convertReplayImage();
   QPainter painter(localImage);
@@ -194,7 +202,18 @@ void editorScene::flattenImage()
 
 void editorScene::convertReplayImage()
 {
-  QImage im(galMW->getLastRxImage());
+    QString fn;
+    QImage im;
+    fn=galleryWidgetPtr->getLastRxImage();
+    if(fn.right(4).toUpper()==".JP2")
+    {
+        im=readJP2Image(fn);
+    }
+    else
+    {
+        im.load(fn);
+    }
+   if(im.isNull()) return;
   //itemBase *it;
   foreach(QGraphicsItem *t,items())
     {
@@ -216,79 +235,80 @@ void editorScene::convertText()
       if(t->type()==itemBase::TEXT)
         {
           itemText *itt=qgraphicsitem_cast<itemText *>(t);
-          itt->setText(textConversion(itt->text()));
+//          itt->setText(textConversion(itt->text()));
+          itt->setText(mexp.convert(itt->text()));
         }
     }
 }
 
-QString editorScene::textConversion(QString str)
-{
-  int i,j;
-  QChar c;
-  convertedText.clear();
-  bool special=FALSE;
-  for (i=0;i<str.length();i++)
-  {
-    if (special)
-      {
-        special=FALSE;
-        c=str.at(i);
-        if(c=='%')
-        {
-          convertedText.append('%');
-          continue;
-        }
-        for (j=0;j<convertList.count();j++)
-          {
-            if(c==convertList.at(j).tag)
-            {
-              convertedText.append(convertList.at(j).replacement);
-            }
-          }
-      }
-    else
-      {
-        if(str.at(i)!='%') convertedText.append(str.at(i));
-        else special=TRUE;
-      }
-  }
-  addToLog("converted text: "+convertedText,DBEDIT);
-  return convertedText;
-}
+//QString editorScene::textConversion(QString str)
+//{
+//  int i,j;
+//  QChar c;
+//  convertedText.clear();
+//  bool special=false;
+//  for (i=0;i<str.length();i++)
+//  {
+//    if (special)
+//      {
+//        special=false;
+//        c=str.at(i);
+//        if(c=='%')
+//        {
+//          convertedText.append('%');
+//          continue;
+//        }
+//        for (j=0;j<convertList.count();j++)
+//          {
+//            if(c==convertList.at(j).tag)
+//            {
+//              convertedText.append(convertList.at(j).replacement);
+//            }
+//          }
+//      }
+//    else
+//      {
+//        if(str.at(i)!='%') convertedText.append(str.at(i));
+//        else special=true;
+//      }
+//  }
+//  addToLog("converted text: "+convertedText,LOGEDIT);
+//  return convertedText;
+//}
 
 
 
 bool editorScene::save(QFile &f,bool templ)
 {
-	QImage im(border.width(),border.height(),QImage::Format_ARGB32);
-	setSceneRect(border);
-  addToLog(QString("editorscene:save %1 x %2").arg(sceneRect().width()).arg(sceneRect().height()),DBEDIT);
-	if(!templ)
-		{
-			QPainter painter(&im);
-      painter.setRenderHint(QPainter::Antialiasing);
-      render(&painter);
-			im.save(&f,"PNG");
-			return TRUE;
-		}
-	if(!f.open(QIODevice::WriteOnly)) return FALSE;
-	QDataStream str(&f);
-	str.setVersion(QDataStream::Qt_4_4);
-	 // Header with a "magic number" and a version
- 	str << (quint32) MAGICNUMBER;
-  str <<  CONFIGVERSION;
-  str << (quint16) QDataStream::Qt_4_4;
-	itemBase *it;
-	foreach(QGraphicsItem *t,items())
-		{
-			it=qgraphicsitem_cast<itemBase *>(t);
-      if(t->type()>itemBase::BASE)
-				{
-					it->save(str);
-				}
-		}
-	f.close();
-	return TRUE;
+    QImage im(border.width(),border.height(),QImage::Format_ARGB32_Premultiplied);
+    setSceneRect(border);
+    addToLog(QString("editorscene:save %1 x %2").arg(sceneRect().width()).arg(sceneRect().height()),LOGEDIT);
+    if(!templ)
+    {
+        QPainter painter(&im);
+        painter.setRenderHint(QPainter::Antialiasing);
+        render(&painter);
+        im.save(&f,"PNG");
+        return true;
+    }
+    if(!f.open(QIODevice::WriteOnly)) return false;
+    QDataStream str(&f);
+    str.setVersion(QDataStream::Qt_4_4);
+    // Header with a "magic number" and a version
+    str << (quint32) MAGICNUMBER;
+    str <<  CONFIGVERSION;
+    str << (quint16) QDataStream::Qt_4_4;
+    itemBase *it;
+    foreach(QGraphicsItem *t,items())
+    {
+        it=qgraphicsitem_cast<itemBase *>(t);
+        if(t->type()>itemBase::BASE)
+        {
+            it->save(str);
+        }
+    }
+    f.close();
+    return true;
 }
 
 void editorScene::setMode(eMode m)
@@ -310,23 +330,23 @@ void editorScene::apply(changeFlags cf)
 	foreach(QGraphicsItem *t,selectedItems())
 		{
 				it=qgraphicsitem_cast<itemBase *>(t);
-			if(cf & FILLCOLOR)
+      if(cf & DFILLCOLOR)
 				{
 				 	it->setBrush(fillColor);
 				}
-			if(cf & LINECOLOR)
+      if(cf & DLINECOLOR)
 				{
 					p=it->pen();
 					p.setColor(lineColor);
 				 it->setPen(p);
 				}
-			if(cf & PEN)
+      if(cf & DPEN)
 				{
 					p=it->pen();
 					p.setWidth(penWidth);
 					it->setPen(p);
 				}
-			if(cf & GRADIENT)
+      if(cf & DGRADIENT)
 				{
 					gradientDialog gd;
 					sgradientParam tmp;
@@ -334,15 +354,15 @@ void editorScene::apply(changeFlags cf)
 					it->setGradient(tmp);
 					it->update();
 				}
-			if(cf & TRANSFORM)
+      if(cf & DTRANSFORM)
 				{
 					it->setTransform(rotate,hShear,vShear);
 				}
 			if(t->type()==itemBase::TEXT)
 				{
 					itemText *itt=qgraphicsitem_cast<itemText *>(t);
-					if(cf & FONT) itt->setFont(font);
-					if(cf & TEXT) itt->setText(text);
+          if(cf & DFONT) itt->setFont(font);
+          if(cf & DTEXT) itt->setText(text);
 				}
 			it->update();
 		}
@@ -387,7 +407,7 @@ void editorScene::setImage(QImage *im)
 	item->setRect(0,0,im->width(),im->height());
 	itemSetup(item);
 	item->setPos(QPointF(0,0));
-	item->setSelected(TRUE);
+	item->setSelected(true);
 	emit changeSize(im->width(),im->height());
 }
 
@@ -492,6 +512,7 @@ void editorScene::slotCopy()
 {
 	itemBase *item;
 	if((!pasted) &&(copyItem!=NULL)) delete copyItem;
+  if(selectedItems().isEmpty()) return; // nothing to do
 	item=qgraphicsitem_cast<itemBase *>(selectedItems().first());
 	makeCopy(item);
 }
@@ -525,7 +546,7 @@ void editorScene::makeCopy(itemBase *it)
 		}
 	copyItem->setParam(item->getParam());
 	copyItem->setPos(item->pos()+QPointF(10,10));
-	pasted=FALSE;
+	pasted=false;
 }
 
 void editorScene::slotPaste()
@@ -534,7 +555,7 @@ void editorScene::slotPaste()
 		copyItem->setZValue(zMax+1);
 		zMax+=1;
 		addItem(copyItem);
-		pasted=TRUE,
+		pasted=true,
 		makeCopy(copyItem);
 		clearSelection();
 }
@@ -542,10 +563,11 @@ void editorScene::slotPaste()
 void editorScene::slotExpand()
 {
 	itemBase *it;
+  if(selectedItems().isEmpty()) return; // nothing to do
 	foreach(QGraphicsItem *t,selectedItems())
 		{
 			it=qgraphicsitem_cast<itemBase *>(t);
-			if(it->type()!=TEXT)
+      if(it->type()!=itemBase::TEXT)
 				{
 					it->setRect(border);
 					it->setPos(0,0);
@@ -555,7 +577,12 @@ void editorScene::slotExpand()
 
 void editorScene::slotChangeText()
 {
+  if(selectedItems().isEmpty()) return; // nothing to do
 	itemText *item=qgraphicsitem_cast<itemText *>(selectedItems().first());
+  if(!item)
+    {
+      return;
+    }
 	if(item->type()!=itemBase::TEXT) return;
 	QDialog d(0);
 	Ui::textForm t;
@@ -570,6 +597,7 @@ void editorScene::slotChangeText()
 
 void editorScene::slotDeleteItem()
 {
+  if(selectedItems().isEmpty()) return; // nothing to do
 	foreach(QGraphicsItem *t,selectedItems())
 		{
 			removeItem(t);
@@ -581,10 +609,11 @@ void editorScene::slotDeleteItem()
 void editorScene::slotLock()
 {
 	itemBase *it;
+  if(selectedItems().isEmpty()) return; // nothing to do
 	foreach(QGraphicsItem *t,selectedItems())
 		{
 			it=qgraphicsitem_cast<itemBase *>(t);
-			it->setLocked(TRUE);
+			it->setLocked(true);
 		}
 }
 
@@ -592,16 +621,18 @@ void editorScene::slotUnlock()
 {
 	
 	 itemBase *it;
+   if(selectedItems().isEmpty()) return; // nothing to do
 	foreach(QGraphicsItem *t,selectedItems())
 		{
 			it=qgraphicsitem_cast<itemBase *>(t);
-			it->setLocked(FALSE);
+			it->setLocked(false);
 		}
 }
 
 
 void editorScene::slotBringToFront()
 {
+  if(selectedItems().isEmpty()) return; // nothing to do
 	foreach(QGraphicsItem *t,selectedItems())
 		{
 			zMax+=1;
@@ -612,6 +643,7 @@ void editorScene::slotBringToFront()
 
 void editorScene::slotSendToBack()
 {
+  if(selectedItems().isEmpty()) return; // nothing to do
 	foreach(QGraphicsItem *t,selectedItems())
 		{
 			t->setZValue(0.5);
@@ -622,6 +654,7 @@ void editorScene::slotSendToBack()
 
 void editorScene::slotSendBackward()
 {
+  if(selectedItems().isEmpty()) return; // nothing to do
 	foreach(QGraphicsItem *t,selectedItems())
 		{
 			t->setZValue(t->zValue()-1.5);
@@ -631,6 +664,7 @@ void editorScene::slotSendBackward()
 
 void editorScene::slotSendForward()
 {
+  if(selectedItems().isEmpty()) return; // nothing to do
 	foreach(QGraphicsItem *t,selectedItems())
 		{
 			t->setZValue(t->zValue()+1.5);
@@ -655,8 +689,8 @@ void editorScene::optimizeDepth()
           it->setZValue(zMax-i);
 					i+=1;
 				}
-      addToLog(QString("optimize_1 type=%1 pos=%2,%3 resctPos=%4,%5").arg(it->getTypeStr()).arg(t->pos().x()).arg(t->pos().y()).arg(it->rect().x()).arg(it->rect().y()),DBEDIT);
-      addToLog(QString("Boundingrect t=%1 %2,%3 %4,%5").arg(it->getTypeStr()).arg(it->boundingRect().x()).arg(it->boundingRect().y()).arg(it->boundingRect().width()).arg(it->boundingRect().height()),DBEDIT);
+      addToLog(QString("optimize_1 type=%1 pos=%2,%3 resctPos=%4,%5").arg(it->getTypeStr()).arg(t->pos().x()).arg(t->pos().y()).arg(it->rect().x()).arg(it->rect().y()),LOGEDIT);
+      addToLog(QString("Boundingrect t=%1 %2,%3 %4,%5").arg(it->getTypeStr()).arg(it->boundingRect().x()).arg(it->boundingRect().y()).arg(it->boundingRect().width()).arg(it->boundingRect().height()),LOGEDIT);
     }
   //optimize position
 
@@ -671,26 +705,47 @@ void editorScene::addBorder(int w,int h)
   itemSetup(borderItemPtr);
   borderItemPtr->setPos(0,0);
   borderItemPtr->setRect(0,0,w,h);
+  border=QRectF(0,0,w,h);
   slotSendToBack();
 }
 
 void editorScene::overlay(QImage *ima)
 {
   clearSelection();
-  addToLog(QString("overlay: before number of items: %1").arg(items().count()),DBEDIT);
-  itemBase *item;
-  item=new itemImage(contextMenu);
-  item->setImage(*ima);
-  itemSetup(item);
-  item->setPos(0,0);
-  item->setRect(0,0,ima->width(),ima->height());
-  item->setSelected(TRUE);
-  slotSendToBack();
-  clearSelection();
-  addToLog(QString("overlay: after number of items: %1").arg(items().count()),DBEDIT);
-  flattenImage();
+//  addToLog(QString("overlay: before number of items: %1").arg(items().count()),LOGEDIT);
+//  itemBase *item;
+//  item=new itemImage(contextMenu);
+//  item->setImage(*ima);
+//  itemSetup(item);
+//  item->setPos(0,0);
+//  item->setRect(0,0,ima->width(),ima->height());
+//  item->setSelected(true);
+//  slotSendToBack();
+//  clearSelection();
+//  addToLog(QString("overlay: after number of items: %1").arg(items().count()),LOGEDIT);
+ // flattenImage(ima->width(),ima->height());
+  setSceneRect(border);
+//  border=sceneRect();
+  if (localImage!=NULL) delete localImage;
+  localImage=new QImage(ima->copy());
+//  localImage=ima;
+  convertText();
+  convertReplayImage();
+  QPainter painter(localImage);
+  painter.setRenderHint(QPainter::Antialiasing);
+//  setSceneRect(0,0,localImage->width(),localImage->height());
+//  localImage->fill(0);
+  render(&painter,QRectF(),QRectF(),Qt::IgnoreAspectRatio);
+}
+
+
+void editorScene::addConversion(QChar tag,QString value,bool clear)
+{
+  if(clear) mexp.clear();
+  mexp.addConversion(tag,value);
 }
 
 
 
 
+
